@@ -10,6 +10,7 @@ import logging
 import argparse
 import re
 import glob
+import textwrap
 import shlex
 import hashlib
 import json
@@ -152,6 +153,7 @@ class Argument():
 
 class StrArgument(Argument):
     HTML_TYPE = "text"
+    HTML_HELP = "Enter plain string"
 
     def argparse_kvsargs(self):
         return {**super().argparse_kvsargs(), **{"action": "store"}}
@@ -159,14 +161,17 @@ class StrArgument(Argument):
 
 class UrlArgument(StrArgument):
     HTML_TYPE = "url"
+    HTML_HELP = "Enter URL string"
 
 
 class MultilineStrArgument(StrArgument):
     HTML_TYPE = "textarea"
+    HTML_HELP = "Enter multiline string"
 
 
 class ChoiceArgument(Argument):
     HTML_TYPE = "select"
+    HTML_HELP = "Select one choice"
 
     def value(self):
         value = super().value()
@@ -186,6 +191,7 @@ class ChoiceArgument(Argument):
 class IntArgument(StrArgument):
     VALUE_TYPE = int
     HTML_TYPE = "number"
+    HTML_HELP = "Enter integer number"
 
     def argparse_kvsargs(self):
         return {**super().argparse_kvsargs(), **{"type": int}}
@@ -194,6 +200,7 @@ class IntArgument(StrArgument):
 class BoolArgument(ChoiceArgument):
     VALUE_TYPE = bool
     HTML_TYPE = "checkbox"
+    HTML_HELP = "Check box to activate"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, choices=[True, False], default=False, **kwargs)
@@ -238,6 +245,7 @@ class _ExtendAction(argparse._AppendAction):  # pylint: disable=protected-access
 class ListArgument(StrArgument):
     VALUE_TYPE = list
     SEPARATOR = ","
+    HTML_HELP = "Enter list as comma-separated strings"
 
     @classmethod
     def s2v(cls, str_value):
@@ -321,7 +329,7 @@ class UploadOptions(StrArgument):
                          default="lintian-mode=ignore",
                          doc=("List of upload options, separated by '|':\n"
                               "\n"
-                              f"{changes.Upload.Options.usage()}"),
+                              f"{textwrap.indent(changes.Upload.Options.usage(), '  ')}"),
                          **kwargs)
 
 
@@ -352,7 +360,7 @@ class Call():
     AUTH = config.AUTH_NONE
     NEEDS_RUNNING_DAEMON = False
     CONFIRM = False
-    CUSTOM_HTML_SNIPPET = None
+    RESULT_DESC = ""
 
     @classmethod
     def name(cls):
@@ -360,7 +368,11 @@ class Call():
 
     @classmethod
     def doc(cls):
-        return inspect.getdoc(cls)
+        doc = inspect.getdoc(cls)
+        if cls.RESULT_DESC:
+            doc += f"\n\nJSON result description:\n\n{textwrap.indent(cls.RESULT_DESC, '  ')}"
+        doc += f"\n\nAuthorization: {cls.AUTH}"
+        return doc
 
     @classmethod
     def doc_title(cls):
@@ -488,16 +500,6 @@ class Call():
         self._run()
 
 
-def _pimpdoc(result_doc=None):
-    def pimp(cls):
-        indent = "    "  # Be sure to keep the same indent like a top level class doc (4 chars) for all lines, so inspect.getdoc() will properly unindent.
-        if result_doc:
-            cls.__doc__ += "\n\n" + "\n".join([f"{indent}{line}" for line in result_doc]) + "\n"
-        cls.__doc__ += f"\n{indent}Authorization: {cls.AUTH}\n"
-        return cls
-    return pimp
-
-
 #: Option shortcuts (use as mixin)
 class _Admin:
     AUTH = config.AUTH_ADMIN
@@ -537,23 +539,24 @@ class _Maintenance:
 
 
 #: API calls
-@_pimpdoc()
 class Status(Call):
     """
-    Get the status of this instance
-
-    JSON Result:
-
-    * version     : mini-buildd's version.
-    * identity    : Instance identity.
-    * url         : Instance URL (HTTP).
-    * incoming_url: Incoming URL (currently FTP).
-    * load        : Instance's (0 =< load <= 1). If negative, the instance is stopped.
-    * chroots     : Active chroots.
-    * remotes     : Active or auto-reactivatable remotes.
-    * [repositories: Simplified structural representation of all repositories]
+    Get status of this instance
     """
 
+    RESULT_DESC = """\
+{
+  "version": mini-buildd's version
+  "identity": Instance identity
+  "url": Instance URL (HTTP)
+  "incoming_url": Incoming URL (currently FTP)
+  "load": Instance's (0 =< load <= 1). If negative, the instance is stopped
+  "chroots": List of active chroots
+  "remotes": Active or auto-reactivatable remotes
+  ["repositories": Simplified structural representation of all repositories]
+}
+"""
+
     @classmethod
     def iarguments(cls):
         yield BoolArgument(["--with-repositories", "-R"], doc="Also list all repositories (may be longish)")
@@ -572,7 +575,6 @@ class Status(Call):
             self.result["repositories"] = {r.identity: r.mbd_json() for r in util.models().Repository.objects.all()}
 
 
-@_pimpdoc()
 class PubKey(Call):
     """
     Get public key
@@ -586,16 +588,11 @@ class PubKey(Call):
         self.result = util.daemon().gnupg.pub_key
 
 
-@_pimpdoc()
 class DputConf(Call):
     """
     Get recommended dput config snippet
 
-    Usually, this is for integration in your personal ~/.dput.cf.
-
-    .. tip:: Can I add a custom dput config snippet?
-
-      Just put your custom snippet in ``~mini-buildd/etc/dput.cf``. If that file exists, it will be added to the output of :apicall:`dput_conf`.
+    Usually, this is for integration in your personal ``~/.dput.cf``.
     """
 
     def _run(self):
@@ -609,12 +606,11 @@ class DputConf(Call):
             pass
 
 
-@_pimpdoc()
 class SourcesList(Call):
     """
     Get sources.list (apt lines)
 
-    Usually, this output is put to a file like '/etc/sources.list.d/mini-buildd-xyz.list'.
+    Usually, this output is put to a file like ``/etc/apt/sources.list.d/mini-buildd-xyz.list``.
     """
 
     @classmethod
@@ -622,12 +618,12 @@ class SourcesList(Call):
         yield Codenames(["--codenames", "-C"], default=values.Name.ALL_CODENAMES)
         yield Repositories(["--repositories", "-R"], default=values.Name.ALL_REPOSITORIES)
         yield ListArgument(["--suites", "-S"], default=values.Name.ALL_SUITES, choices=values.Name.ALL_SUITES, doc="Suite names (like 'stable', 'unstable')")
-        yield ListArgument(["--types", "-T"], default=["deb"], choices=["deb", "deb-src"], doc="Types of apt lines")
+        yield ListArgument(["--types", "-T"], default=["deb"], choices=["deb", "deb-src"], doc="Types of apt lines ('deb', 'deb-src')")
         yield StrArgument(["--options", "-O"], default="", doc="Apt line options ('deb[-src] [<options>] ...'). See 'man 5 source.list'")
         yield Rollbacks(["--rollbacks", "-r"])
-        yield StrArgument(["--snapshot", "-P"], default="", doc="Select a repository snapshot")
+        yield StrArgument(["--snapshot", "-P"], default="", doc="Select a repository snapshot (see ``snapshot_ls``)")
         yield StrArgument(["--mirror", "-M"], default="", doc="URL of a mirror to use (instead of mini-buildd's native URL)")
-        yield BoolArgument(["--with-extra", "-X"], doc="Also list extra sources needed")
+        yield BoolArgument(["--with-extra", "-X"], doc="Also list extra sources (non-base sources used for building)")
         yield BoolArgument(["--with-comment", "-D"], doc="Add comment line above apt line")
 
     def _run(self):
@@ -650,7 +646,6 @@ class SourcesList(Call):
                                        with_comment=self.args["with_comment"].value())
 
 
-@_pimpdoc()
 class Ls(Call):
     """
     List source package
@@ -674,7 +669,6 @@ class Ls(Call):
                 self.result[repository.identity] = ls
 
 
-@_pimpdoc()
 class Show(Ls):
     """
     Show source package
@@ -709,7 +703,6 @@ class Debdiff(Call):
         self.result = call.Call(["debdiff", repositories_path.join(dsca), repositories_path.join(dscb)]).stdout
 
 
-@_pimpdoc()
 class Migrate(_Staff, _Confirm, Call):
     """
     Migrate source package
@@ -737,7 +730,6 @@ class Migrate(_Staff, _Confirm, Call):
                                        version=version if version else None)
 
 
-@_pimpdoc()
 class Remove(_Admin, _Confirm, Call):
     """
     Remove source package
@@ -763,22 +755,14 @@ class Remove(_Admin, _Confirm, Call):
                                       without_rollback=self.args["without_rollback"].value())
 
 
-PORT_RESULT_DOC = [
-    "JSON Result (dict):",
-    "  list of dicts of all items uploaded",
-]
-
-
-@_pimpdoc(PORT_RESULT_DOC)
 class Port(_Staff, _Running, _Confirm, Call):
     """
     Port internal source package
 
-    An internal 'port' is a no-changes (i.e., only the changelog
-    will be adapted) rebuild of the given locally-installed
-    package.
+    An internal 'port' is a no-changes (i.e., only the changelog will be adapted)
+    rebuild of the given locally-installed package.
 
-    When from-distribution equals to_distribution, a rebuild will be done.
+    When ``from_distribution`` equals ``to_distribution``, a rebuild will be done.
     """
 
     @classmethod
@@ -800,13 +784,12 @@ class Port(_Staff, _Running, _Confirm, Call):
                                             options=self.args["options"].value().split("|")))
 
 
-@_pimpdoc(PORT_RESULT_DOC)
 class PortExt(_Staff, _Running, _Confirm, Call):
     """
     Port external source package
 
-    An external 'port' is a no-changes (i.e., only the changelog
-    will be adapted) rebuild of any given source package.
+    An external 'port' is a no-changes (i.e., only the changelog will be adapted)
+    rebuild of any given source package.
     """
 
     @classmethod
@@ -825,22 +808,18 @@ class PortExt(_Staff, _Running, _Confirm, Call):
                                                 allow_unauthenticated=self.args["allow_unauthenticated"].value()))
 
 
-@_pimpdoc()
 class Retry(_Staff, _Running, _Confirm, Call):
     """
     Retry a previously failed source package
-
-    JSON Result:
-
-    * changes: Changes file that has been re-uploaded.
     """
 
+    RESULT_DESC = "Changes file name that has been re-uploaded"
     BKEY_FORMAT = "<source>/<version>/<timecode>/source[ <arch>]"
     BKEY_REGEX = re.compile(r"[^/]+/[^/]+/[^/]+/[^/]+")
 
     @classmethod
     def iarguments(cls):
-        yield StrArgument(["bkey"], choices=values.Name.LAST_FAILED_BKEYS, doc=f"Package bkey ('{cls.BKEY_FORMAT}') of a past packaging try; 'extra.bkey' of a FAILED packaging event")
+        yield StrArgument(["bkey"], choices=values.Name.LAST_FAILED_BKEYS, doc=f"Package bkey (``{cls.BKEY_FORMAT}``) of a past packaging try (see ``extra.bkey`` of a failed PACKAGING event)")
 
     def _run(self):
         bkey = self.args["bkey"].value()
@@ -870,22 +849,22 @@ class Retry(_Staff, _Running, _Confirm, Call):
         self.result = os.path.basename(changes_filename)
 
 
-@_pimpdoc()
 class Cancel(_Staff, _Running, _Confirm, Call):
     """Cancel an ongoing package build"""
 
     @classmethod
     def iarguments(cls):
-        yield StrArgument(["bkey"], choices=values.Name.CURRENT_BUILDS, doc="Build key ('<source>/<version>/<timecode>/<arch>' of ongoing builds; 'extra.bkey' in events)")
+        yield StrArgument(["bkey"], choices=values.Name.CURRENT_BUILDS, doc="Build key (``<source>/<version>/<timecode>/<arch>`` of ongoing builds; see ``extra.bkey`` in BUILDING events)")
 
     def _run(self):
         util.daemon().builder.cancel(self.args["bkey"].value(), f"{self.request.user}")
         self.result = self.args["bkey"].value()
 
 
-@_pimpdoc()
 class SetUserKey(_Login, _Confirm, Call):
-    """Set a user's GnuPG public key"""
+    """
+    Set a user's GnuPG public key
+    """
 
     @classmethod
     def category(cls):
@@ -914,9 +893,10 @@ class SetUserKey(_Login, _Confirm, Call):
         LOG.warning(f"Uploader profile changed: {uploader} (must be (re-)activated by mini-buildd staff before you can actually use it)")
 
 
-@_pimpdoc()
 class Subscribe(_Login, Call):
-    """Subscribe to (email) notifications"""
+    """
+    Subscribe to (email) notifications
+    """
 
     @classmethod
     def iarguments(cls):
@@ -940,9 +920,10 @@ class Subscribe(_Login, Call):
         self.result = {"created": created, "source": s.package, "distribution": s.distribution}
 
 
-@_pimpdoc()
 class Unsubscribe(Subscribe):
-    """Unsubscribe from (email) notifications"""
+    """
+    Unsubscribe from (email) notifications
+    """
 
     def _run(self):
         self.result = []
@@ -952,7 +933,6 @@ class Unsubscribe(Subscribe):
                 self.result.append({"source": s.package, "distribution": s.distribution})
 
 
-@_pimpdoc()
 class RemakeChroots(_Admin, _Confirm, _Maintenance, Call):
     """
     Remake chroots
@@ -980,7 +960,6 @@ class RemakeChroots(_Admin, _Confirm, _Maintenance, Call):
                 self.result[key] = f"Failed: {util.e2http(e)}"
 
 
-@_pimpdoc()
 class Start(_Admin, _Confirm, Call):
     """
     Start Daemon (accept incoming)
@@ -992,7 +971,6 @@ class Start(_Admin, _Confirm, Call):
         util.daemon().mbd_start()
 
 
-@_pimpdoc()
 class Stop(_Admin, _Confirm, Call):
     """
     Stop Daemon (stop accepting incoming)
@@ -1006,9 +984,10 @@ class Stop(_Admin, _Confirm, Call):
         util.daemon().mbd_stop()
 
 
-@_pimpdoc()
 class Wake(_Staff, _Confirm, Call):
-    """Wake a remote instance"""
+    """
+    Wake a remote instance
+    """
 
     @classmethod
     def iarguments(cls):
@@ -1021,7 +1000,6 @@ class Wake(_Staff, _Confirm, Call):
         self.result = remote.mbd_get_status(wake=True, wake_sleep=self.args["sleep"].value(), wake_attempts=self.args["attempts"].value())
 
 
-@_pimpdoc()
 class Handshake(Call):
     """
     Check if signed message matches a remote, reply our signed message on success
@@ -1050,7 +1028,6 @@ class Handshake(Call):
         raise util.HTTPBadRequest(f"GnuPG handshake failed: No remote for public key on {util.http_endpoint()}")
 
 
-@_pimpdoc()
 class Cronjob(_Admin, _Confirm, Call):
     """
     Run a cron job now (out of schedule)
@@ -1065,7 +1042,6 @@ class Cronjob(_Admin, _Confirm, Call):
         self.result[job.id()] = job.run()
 
 
-@_pimpdoc()
 class Uploaders(_Admin, _Running, Call):
     """
     Get upload permissions for repositories
@@ -1084,13 +1060,9 @@ class Uploaders(_Admin, _Running, Call):
                 self.result[r.identity]["uploaders"] = gpg.get_pub_keys_infos()
 
 
-@_pimpdoc()
 class SnapshotLs(_Running, Call):
     """
     Get list of repository snapshots for a distribution
-
-    JSON Result (dict):
-      dict: <distribution>: [snasphot,...]: List of snapshots for the given distribution
     """
 
     @classmethod
@@ -1102,9 +1074,10 @@ class SnapshotLs(_Running, Call):
         self.result = diststr2repository(diststr).mbd_reprepro.get_snapshots(diststr)
 
 
-@_pimpdoc()
 class SnapshotCreate(_Admin, _Confirm, SnapshotLs):
-    """Create a repository snapshot"""
+    """
+    Create a repository snapshot
+    """
 
     @classmethod
     def iarguments(cls):
@@ -1117,23 +1090,23 @@ class SnapshotCreate(_Admin, _Confirm, SnapshotLs):
 
 
 class SnapshotDelete(SnapshotCreate):
-    """Delete a repository snapshot"""
+    """
+    Delete a repository snapshot
+    """
 
     def _run(self):
         diststr = self.args["distribution"].value()
         diststr2repository(diststr).mbd_reprepro.del_snapshot(diststr, self.args["name"].value())
 
 
-@_pimpdoc()
 class Debmirror(_Admin, _Confirm, Call):
     """
     Make local partial repository mirror via :debpkg:`debmirror`
 
-    This may be useful if you plan on publishing a stripped-down (f.e.,
-    only certain repos, only ``stable``, omit rollbacks) variant of your
-    repo somewhere remote.
+    This may be useful if you plan on publishing a stripped-down (f.e., only certain repos,
+    only ``stable``, omit rollbacks) variant of your repo somewhere remote.
 
-    .. error:: debmirror (:debbug:`819925`): ``apt update`` fails on ``experimental`` suites (contents not mirrored)
+    .. error:: debmirror: ``apt update`` fails on ``experimental`` suites (contents not mirrored) (:debbug:`819925`)
 
       This happens only on systems where APT is configured to download contents, most likely just because
       ``apt-file`` is installed. So, the easiest workaround is::
@@ -1141,8 +1114,8 @@ class Debmirror(_Admin, _Confirm, Call):
         apt purge apt-file
 
       Closest to an actual fix is to install debmirror variant '+abfixes' from
-      ``Hellfield Archive``, where the "bug" has been fixed (Dec 2022: at least
-      available for bullseye).
+      ``Hellfield Archive``, where the "bug" has been fixed (June 2023: at least
+      available for bullseye and bookworm).
     """
 
     PROGRAM = "/usr/bin/debmirror"
@@ -1192,12 +1165,11 @@ class Debmirror(_Admin, _Confirm, Call):
             call.Call(debmirror).check()
 
 
-@_pimpdoc(PORT_RESULT_DOC)
 class KeyringPackages(_Admin, _Running, _Confirm, Call):
     """
     Build keyring packages
 
-    .. tip:: **keyring-packages**: No compat for urold (``apt-key add``)
+    .. note:: **keyring-packages**: No compat for urold (``apt-key add``)
 
       Since ``2.x``, keyring packages will use ``/etc/apt/trusted.gpg.d/<foo>.gpg``, not deprecated ``apt-key add <foo>``.
 
@@ -1235,9 +1207,10 @@ class KeyringPackages(_Admin, _Running, _Confirm, Call):
                         repository.mbd_package_migrate(upload["source"], distribution, suite, full=True, version=upload["version"])
 
 
-@_pimpdoc(PORT_RESULT_DOC)
 class TestPackages(_Admin, _Running, _Confirm, Call):
-    """Build test packages"""
+    """
+    Build test packages
+    """
 
     __TEMPLATES = ["mbd-test-archall", "mbd-test-cpp", "mbd-test-ftbfs"]
 
@@ -1271,7 +1244,6 @@ class TestPackages(_Admin, _Running, _Confirm, Call):
                         LOG.debug(f"Test package result: {event}")
 
 
-@_pimpdoc()
 class Setup(_Admin, _Confirm, _Maintenance, Call):
     """
     Create, update or inspect your setup
@@ -1330,7 +1302,7 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
         yield ListArgument(["--chroots"],
                            default=[],
                            doc=("Chroots to setup (uses same syntax as ``--distributions``).\n\n"
-                                ".. tip:: With :debpkg:`qemu-user-static` installed, you also have seamless access to foreign archs (albeit with a speed penalty)"),
+                                ".. tip:: Install :debpkg:`qemu-user-static` for seamless access to foreign archs (albeit with a speed penalty)"),
                            header="Chroots")
         yield BoolArgument(["--chroots-from-sources"], doc="Auto-add chroots for all given ``sources`` codenames with native architectures")
         yield ChoiceArgument(["--chroots-backend", "-C"], default=values.Name.DEFAULT_CHROOT_BACKEND, choices=["Dir", "File", "LVM", "LoopLVM", "BtrfsSnapshot"], doc="Chroot backend to use")
@@ -1339,37 +1311,49 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
 
     class Report(dict):
         SEVERITY_MESSAGES = {
-            None: "Not run yet",
             0: "No issues found",
             1: "Notable issues found",
-            2: "Warning issues found",
-            3: "Error issues found",
+            2: "Warnings found",
+            3: "Errors found",
         }
 
-        def __init__(self):
-            self.severity, self.timestamp = None, None
-
-        def add(self, key, severity, quantity, search=""):
+        def _add(self, key, severity, quantity, search=""):
             self[key] = {
                 "severity": severity if quantity > 0 else 0,
                 "quantity": quantity,
                 "search": search,
             }
 
-        def done(self):
+        def __init__(self, result):
+            self._add("Total instances traversed", 0, len(result["instances"]))
+            self._add("Differences from setup", 2, sum((1 for i in result["instances"] if i["diff"]["setup"]["diff"])), '"setup diff"')
+            self._add("Differences from model defaults", 1, sum((1 for i in result["instances"] if i["diff"]["model"]["diff"])), '"model diff"')
+            for status, severity in Setup.Instance.REPORT_STATUS.items():
+                self._add(f"{status.name.capitalize()} instances", severity, sum((1 for i in result["instances"] if i["status"] == status.name)), status.name)
             self.severity = max(i["severity"] for i in self.values())
             self.timestamp = util.Datetime.now()
 
         def __str__(self):
-            return f"{self.SEVERITY_MESSAGES[self.severity]} on {self.timestamp}"
+            return f"Inspection: {self.SEVERITY_MESSAGES[self.severity]} (on {self.timestamp})"
 
-        #: Workaround for HTML template only
-        def severity_class(self):
-            return f"mbd-severity{self.severity}"
+    @classmethod
+    def report_path(cls):
+        return config.ROUTES["log"].path.join("inspect.json")
+
+    @classmethod
+    def report_load(cls):
+        with util.fopen(cls.report_path()) as r:
+            report = cls.Report(json.load(r))
+            report.timestamp = util.Datetime.from_path(cls.report_path())
+            return report
+
+    def report_save(self):
+        with util.fopen(self.report_path(), "w") as r:
+            json.dump(self.result, r)
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.report = self.Report()
+        self.report = None
 
     class Instance:
         class Status(enum.Enum):
@@ -1679,6 +1663,12 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
                 diff = super().diff()
                 if self.obj is not None:
                     self._add_m2m_diff(diff, "apt_keys", self.update_args["apt_keys"], [a.mbd_key_id() for a in self.obj.apt_keys.all()])
+
+                    if dist.is_base_source(self.obj.origin, self.obj.codename) and self.obj.codeversion != self.obj.codeversion_override:
+                        diff["model"]["diff"]["codeversion"] = {
+                            "current": self.obj.codeversion,
+                            "target": self.obj.codeversion_override,
+                        }
                 return diff
 
         for codename in (dist.Codename(c) for c in setup["sources"]):
@@ -1820,13 +1810,7 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
             Setup.Instance(self, util.models().Remote, {}, {}, http=ep)
 
         # Report
-        self.report.add("Total instances traversed", 0, len(self.result["instances"]))
-        self.report.add("Differences from setup", 2, sum((1 for i in self.result["instances"] if i["diff"]["setup"]["diff"])), '"setup diff"')
-        self.report.add("Differences from model defaults", 1, sum((1 for i in self.result["instances"] if i["diff"]["model"]["diff"])), '"model diff"')
-        for status, severity in self.Instance.REPORT_STATUS.items():
-            self.report.add(f"{status.name.capitalize()} instances", severity, sum((1 for i in self.result["instances"] if i["status"] == status.name)), status.name)
-
-        self.report.done()
+        self.report = self.Report(self.result)
 
     def _run(self):
         self.__run()
